/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @extends SelectableSymbol
 * @returns {undefined}
 */
var Category = Backbone.Model.extend ({
    
    prefixId: 'c-',                         // Prefix for ids
    defaults: {
        id: null,                           // Default Category id
        text: 'categoria',                  // Default category
        border: new Border(),               // Category border
        backColor: "#FFFFFF",               // Category backcolor
        image: new Image(),                 // Category background image
        font: new Font(),                   // Default category font
        rows: 2,                            // Rows of category
        cols: 4,                            // Cols of category
        symbol: {}                          // Category symbol reference (show in selection panels)
    },
    
    /**
     * Return list of attributes to export in json object
     */
    exportAttributes: function () {
        return new Array('id','text','border','backColor','image','font','rows','cols','symbol','grid');
    },
    
    initialize: function () {
        this.attributes.id = this.attributes.id || _.uniqueId(this.prefixId);   
    },
    
    parse: function (response) {
        
        response.border = new Border(response.border);
        response.image = new Image(response.image);
        response.font = new Font (response.font);
        
        return response;
    },
   
},{ // Prototype methods
  
    /**
     * Add a selectable simbol to the list
     * @param {SelectableSimbol} object
     * @returns {array} List of simbols
     */
    addSelectable: function (object) {
        // TODO implement function
    },

    /**
     * Delete a símbol from the list
     * @param {SelectableSimbol} object
     * @returns {array} List of simbols
     */
    deleteSelectable: function (object) {
        // TODO implement function
    },

    /**
     * Move a simbol to change the order
     * @param {SelectableSimbol} object
     * @param {Integer} position
     * @returns {array} List of simbols
     */
    moveSelectable: function (object, position) {
        // TODO implement function
    }

});


var Categories = MultiSAACCollection.extend ( {
    
    model: Category,
    
    parse: function (o) {
        this.version = o.version;
        return o.items;
    }
});

//@ sourceURL=Category.js
