/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * TextPostion
 * @type Enumeration
 * @description Possible text position in the simbol
 */
var TextPosition = {up: "up", 
                    down: "down"};
                
/**
 * Class SelectableSymbol
 * @description This class must be a abstract class. 
 *              It provides common information for selectable simbol objects in the board
 *              TODO make a function to automatically set unique id for each object
 * @returns {SelectableSymbol}
 */
var SelectableSymbol = Selectable.extend({
    
    defaults: (function() {     // Extend parent defaults
        return _.defaults({
            text: "",                     // Text information
            image: new Image(),           // Image information
            hoverImage: new Image(),      // Image information when the object is focused
            sound: new Sound(),           // Sound information (it's only effective when the object is active)
            video: new Video(),           // Video information (it's only effective when the object is active)
            font: new Font(),             // Font information
            hoverFont: new Font(),        // Font information when the object is focused
            position: TextPosition.down   // Text position
        }, _.result(Selectable.prototype, 'defaults'));
    })(),
    
    parse: function (o) {
        o = SelectableSymbol.__super__.parse(o);
        if (o.image) o.image = new Image(o.image,{parse: true});                  // Parse image if it's setted
        if (o.hoverImage) o.hoverImage = new Image(o.hoverImage,{parse: true});   // Parse hoverImage if it's setted
        if (o.sound) o.sound = new Sound(o.sound,{parse: true});                  // Parse sound if it's setted
        if (o.video) o.video = new Video(o.video,{parse: true});                  // Parse video if it's setted
        if (o.font) o.font = new Font(o.font,{parse: true});                      // Parse font if it's setted
        if (o.hoverFont) o.hoverFont = new Font(o.hoverFont,{parse: true});       // Parse hoverFont if it's setted
        return o;
    }
}, {    // Prototype Methods
    
   TextPosition: TextPosition,        // Include constants as static constant class
   /*
    * Function play
    * @description This function active the object. 
    *              If the object has Sound it will be played, otherwise. if the object has Video it will be played.
    *              If present, selectedImage will be displayed instead of image.
    *              SelectedBorder and selectedBackColor will be used insead of standard properties
    * @returns (nothing)
    */
   play: function() {
       // TODO implement play function
   },
   
   /*
    * Function stop
    * @description This function deactive the object. 
    *              Sound and video will be stopped.
    *              Standard properties will replace selected properties
    * @returns (nothing)
    */
   stop: function() {
       // TODO implement stop function
   }
   
});
