/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

var DirectSelector = Selector.extend ({
    
    /**
     * Events to add for each selectable. Not be confused with object events.
     * Key will be event name on the selectable element
     * Value is the handler method. Methods are from this object
     */ 
    eventsList: {   
        "mouseenter": "focalize",        // Event to focalize (focus) selectable
        "mouseleave": "unfocalize",      // Event to unfocalize (lostFocus) selectable
        "click":      "select"           // Event to select selectable
    },
    
    /**
     * Instance intilizer
     */
    initialize: function () {
        this.prepare();
    },
   
    /**
     * play
     * @description Starts selector behavior.
     *      On succes status will change to PLAYING
     * @param selector (Optional) jQuery selector string specify concret context to apply selector 
     * @return this
     */
    play: function (selector) {
        
        // First thing is set the selector
        this.setSelectorSring(selector || '');
        
        // Apply events to symbols
        this.addEventsToSelection();
        
        // Change status
        this.setStatus(SELECTOR_STATUS.PLAYING);
        return this;
    },
    
    /**
     * stop
     * @descripion Stop selector behavior. On resume it will start from first element
     * @params Nothing
     * @return this
     */
    stop: function () {
        console.log("stop");
        this.setStatus(SELECTOR_STATUS.STOP);
        this.unsetStatus(SELECTOR_STATUS.PLAYING);
        return this;
    },

    /**
     * pause
     * @descripion Pause selector behavior. On resume it will continue from actual element
     * @params Nothing
     * @return this
     */
    pause: function () {
        console.log("pause");
        this.setStatus(SELECTOR_STATUS.PAUSE);
        return this;
    },
    
    /**
     * resume
     * @descripion Resume selector behavior. It will start depends on previous call (stop or pause)
     * @params Nothing
     * @return this
     */
    resume: function () {
        console.log("resume");
        this.unsetStatus(SELECTOR_STATUS.PAUSE | SELECTOR_STATUS.STOP);
        this.setStatus(SELECTOR_STATUS.PLAYING);
        return this;
    },
    
    /**
     * rewind
     * @descripion Restart from first element. ShortHand to Stop an Resume
     * @params Nothing
     * @return this
     */
    rewind: function () {
        console.log("rewind");
        return this.stop().resume();
    },
    
    /**
     * clear
     * @descripion Clear list of elements to apply selector
     * @params Nothing
     * @return this
     */
    clear: function () {
        console.log("clear");
        //this.removeEventsToSelection();
        var status = this.isStatus(SELECTOR_STATUS.PREPARED) ? SELECTOR_STATUS.PREPARED | SELECTOR_STATUS.CLEARED : SELECTOR_STATUS.CLEARED;
        this.changeStatus(status);
        return this;
    },
    
    /**
     * Events handler
     */
    
    /**
     * Focalize event handler
     * @description Trigger focalize event to element
     * @params jQueryEvent
     */
    focalize: function (event) {
        
       // console.log("Selector focalize");
        
        // If status is playing and not is also pause ...
        if (this.isStatus(SELECTOR_STATUS.PLAYING) && 
            ! this.isStatus(SELECTOR_STATUS.PAUSE)) {
        
            // Saving current object
            this.hoverSelectable = event.currentTarget;

            // Trigger event to the element
            $(this.hoverSelectable).trigger("focalize",event);
            
        }
        event.stopImmediatePropagation();
    },
    
    /**
     * UnFocalize event handler
     * @description Trigger unfocalize event to element
     * @params jQueryEvent
     */
    unfocalize: function (event) {
        
       // console.log("Selector unfocalize");
        // If status is playing and not is also pause ...
        if (this.isStatus(SELECTOR_STATUS.PLAYING) && 
            ! this.isStatus(SELECTOR_STATUS.PAUSE)) {
            $(event.currentTarget).trigger("unfocalize",event);
        }
        event.stopImmediatePropagation();
    },
    
    /**
     * Select event handler
     * @description Trigger select event to element
     * @params jQueryEvent
     */
    select: function (event) {
        
      //  console.log("Selector select");
        // If status is playing and not is also pause ...
        if (this.isStatus(SELECTOR_STATUS.PLAYING) && 
            ! this.isStatus(SELECTOR_STATUS.PAUSE)) {
            $(event.currentTarget).trigger("select", event);
        }
        event.stopImmediatePropagation();
    }
    
});

//@ sourceURL=DirectSelector.js