/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


var BBOARD_STATE = {
    STOPPED: 0,
    PLAYING: 1
}

var BlackBoardView = View.extend({
    
    bboardSymbols: new Array(),
    state: BBOARD_STATE.STOPPED,
    
    defaults: {
        templatesID: ["blackBoard","blackBoardEdit"]
    },
    
    initialize: function () {
        this.prepare.apply(this, arguments);
        this.attributes.id = this.attributes.id || "blackboard";
        this.blackBoard = this.board.get('blackBoard');
        this.renderContent();
    },
    
    renderContent: function () {
        this.renderByData('<div id="blackboard" class="ui-block-a blackboard"></div>\n\
                           <div id="blackboardSymbols" class="ui-block-b ui-grid-b"></div>');
        var $id = "#" + this.attributes.id;
        $id = '#blackboard';
        this.$bboard = $('#blackboard');
        this.$bboard.css('background-color',this.attributes.backColor);
        this.border = new BorderView(_.extend(this.attributes.border, { el: $($id )}));
        this.$ctrlSymbols = $('#blackboardSymbols');
        this.eraseSymbol = new SymbolView(_.extend(this.blackBoard.eraseSymbol, { el: this.$ctrlSymbols, className: 'ui-block-c erase'}));
        this.backSymbol = new SymbolView(_.extend(this.blackBoard.backSymbol, { el: this.$ctrlSymbols, className: 'ui-block-b back'}));
        this.playSymbol = new SymbolView(_.extend(this.blackBoard.playSymbol, { el: this.$ctrlSymbols, className: 'ui-block-a play'}));
       
        return this;
    },
    
    renderEdit: function () {
        var controls = $('<button class="ui-icon-edit edit" title="Editar pizarra">Opciones</button>');
        View.renderControls(controls,this.$bboard);
        this.eraseSymbol.renderEdit();
        this.backSymbol.renderEdit();
        this.playSymbol.renderEdit();
    },
    
    events: {
        "add": "addSymbol",
        "select .symbol.selectable.back": "deleteLastSymbol",
        "select .symbol.selectable.erase": "erase",
        "select .symbol.selectable.play": "play",
        "click button.edit": "edit"
    },
    
    edit: function () {
        app.message('TODO: Pantalla para editar la pizarra','todo-dialog'); 
    },
    
    getState: function () {
        return this.state;
    },
    
    setState: function (state) {
        this.state = state;
    },
    
    play: function (event) {
        if (this.getState == BBOARD_STATE.PLAYING) {
            console.log("BlackBoard is playing now");
        } else {
            this.setState(BBOARD_STATE.PLAYING);
            var playIndex = this.playIndex = 0;
            var symbols = this.bboardSymbols;
                
            if (symbols[playIndex])  {
                this.scroll(symbols[playIndex]);
                symbols[playIndex].play(event, _.bind(this.parcialEndPlay,this));
            }
        }
        return false;
    },
    
    parcialEndPlay: function (event) {
        if (this.getState == BBOARD_STATE.STOPPED) {
            console.warn("BlackBoard is stopped now, this event is out of time.");
        } else {
            if (event) event.stopImmediatePropagation();
            this.bboardSymbols[this.playIndex].stop(event,_.bind(this.playNext,this));
        }
        return false;
    },
    
    playNext: function (event) {
        if (this.getState == BBOARD_STATE.STOPPED) {
            console.warn("BlackBoard is stopped now, this event is out of time.");
        } else {
            if (event) event.stopImmediatePropagation();
            this.playIndex++;
            if (this.bboardSymbols[this.playIndex]) {
                this.scroll(this.bboardSymbols[this.playIndex]);
                this.bboardSymbols[this.playIndex].play(event, _.bind(this.parcialEndPlay,this));
            }
            else {
                this.setState(BBOARD_STATE.STOPPED);
            }
        }
        return false;
    },
    
    /**
     * Scroll to symbol
     */
    scroll: function (symbol) {
        var pos = symbol.postion;
        console.log(pos);
        this.$bboard.animate({ scrollTop: pos.top,scrollLeft: pos.left });
    },
    
    /**
    * Add a symbol to the play list
    * @param (event) event
    * @param {symbol} symbol Simbol to add
    * @returns {array} List to play
    */
    addSymbol: function (event, symbol) {
        var bbSymbol = symbol.clone();
        bbSymbol.set('id', 'bb-' + this.bboardSymbols.length);
        var symbol = new SymbolView(_.extend(bbSymbol, { el: this.$bboard}));
        symbol.postion = symbol.$el.position();
        this.bboardSymbols.push(symbol);
        return false;
    },
    
    /**
    * Remove last added simbol
    * @returns {array} List to play
    */
    deleteLastSymbol: function (event) {
        var symbol = this.bboardSymbols.pop();
        if (symbol) {
            symbol.$el.remove();
            symbol.release();
        }
        return false;
    },
    
    /**
    * Clear simbol list
    * @returns {undefined}
    */
    erase: function () {
       var symbol = null;
       while (symbol = this.bboardSymbols.pop()) {
           symbol.stop();
           symbol.$el.remove();
           symbol.release();
       }
       return false;
    }
    
});

//@ sourceURL=BlackBoardView.js