/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

var BoardView = View.extend({
    
    navList: new Array(),       // Array of visiteds categories
    
    defaults: {
        templatesID: ["board","boardEdit"]
    },
    
    initialize: function () {
        this.prepare.apply(this, arguments);
        View.symbols = this.attributes.symbols;
        this.renderContent();
    },
    
    renderBlackBoard: function () {
        if (this.attributes.hasBlackBoard) {
            this.bboard = new BlackBoardView(_.extend(this.attributes.blackBoard, 
                                                    { el: $("#blackboardwrapper"), 
                                                      collection: this.attributes.blackBoard.attributes.list, 
                                                      useTo: this.getUseTo()}));
        }
    },
    
    renderContent: function () {
        // Render board
        var content='<div data-role="header">\n\
                        <button class="ui-btn ui-btn-inline ui-icon-back ui-btn-left ui-btn-icon-right ui-shadow ui-corner-all ui-btn-icon-notext logout">Cerrar tablero</button>\n\
                            <h3>Tablero</h3>\n\
                        <button class="ui-btn ui-btn-inline ui-icon-gear ui-btn-right ui-btn-icon-right ui-shadow ui-corner-all ui-btn-icon-notext edit-enable">Activar edición</button>\n\
                    </div>\n\
                    <div id="wrapper" data-role="content">\n\
                        <div id="boardwrapper">';
        if (this.attributes.hasBlackBoard) { 
            content+='<div id="blackboardwrapper" class="ui-grid-a"></div>';
        }
        content+='          <div id="categorywrapper"></div>\n\
                        </div>\n\
                  </div>';
    
        this.renderByData(content);
        
        // BlackBoard render
        this.renderBlackBoard();
        
        // Category render
        var category = this.selectedCategory || this.attributes.categories.get(this.attributes.defaultCategory.id);
        this.navList.push(category);
        this.renderCategory(category);
    },
    
    renderCategory: function (category) {
        this.navList.push(this.selectedCategory);
        this.categoryView = new CategoryView(_.extend(category, { el: $("#categorywrapper"), useTo: this.getUseTo()}));
        this.selectedCategory = category;
        // Start selector
        this.board.selector.prepare();
        this.board.selector.play();
        event.stopImmediatePropagation();
    },
    
    /**
     * Shows edit interface
     */
    renderEdit: function () {
        $(this.$el).find('.edit-enable').hide();
        var controls = $('<button class="ui-icon-back edit-disable" title="Desactivar edicioón">Desactivar edición</button>\n\
                          <button class="ui-icon-edit edit" title="Editar tablero">Opciones</button>\n\
                        ');
        View.renderControls(controls,this.$el,10);
        this.bboard.renderEdit();
        this.categoryView.renderEdit();
    },
    
    destroy: function () {
        this.bboard.remove();
        this.categoryView.remove();
        this.undelegateEvents();
    },
    
    events: {
        "select .symbol.selectable.back-symbol": "backButtonSelected",
        "select .symbol.selectable.category-symbol": "categorySelected",
        "select .symbol.selectable": "symbolSelected",
        "click button.edit-disable": "disableEdit",
        "click button.edit-enable": "enableEdit",
        "click button.edit": "editBoard",
        "click button.logout": "logout"
    },
    
    /** 
     * Handle select symbol events
     */
    logout: function () {
        bm.exit();
    },
    
    editBoard: function () {
        app.message('TODO: Pantalla de edición de opciones de tablero','todo-dialog');
    },
    
    enableEdit: function () {
        bm.enableEdit();
    },
    
    disableEdit: function () {
        if (bm.disableEdit()) {
            $('.edit-controls').remove();
            $(this.$el).find('.edit-enable').show();
        }
    },
    
    symbolSelected: function (event) {
       
        var attr = this.attributes;
        // Void symbol's can't be selecteds
        if ($(event.currentTarget).hasClass(SymbolView.SYMBOL_VOID_CLASS)) {
            event.stopImmediatePropagation();
        } else if (attr.hasBlackBoard) {
            var symbol = View.symbols.get(event.currentTarget.dataset.idx);
            this.bboard.$el.trigger('add',symbol);
        } else {
            $(event.currentTarget).trigger('play',event);
        }
        
        return false;
    },
    
    backButtonSelected: function (event) {
        var category = this.navList.pop();
        this.categoryView.release();
        // Change selectedCategory and render it
        this.renderCategory(category);
        event.stopImmediatePropagation();
        return false;
    },
    
    categorySelected: function (event) {
        console.log('Changing category ...');
        // Release resources associated to the category
        this.categoryView.release();
        // Change selectedCategory and render it
        var category = this.attributes.categories.get(event.currentTarget.dataset.categoryidx);
        this.renderCategory(category);
        event.stopImmediatePropagation();
        return false;
    },
    
});

//@ sourceURL=BoardView.js