/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

    
var CategoryView = View.extend({
    
    symbols: new Array(),        // List of loaded symbols 
    
    defaults: {
        templatesID: ["category", "categoryEdit"],
        className: "category"
    },
    
    initialize: function () {
        this.prepare.apply(this, arguments);
        this.renderContent();
    },
    
    renderContent: function () {
        var attr = this.attributes;
        
        // Render the category container
        this.renderByData('<div class="'+this.attributes.className+'" id="'+this.attributes.id+'">\n\
                                <h3>'+this.attributes.text+'</h3>\n\
                           </div>');
        
        // Get the rendered container
        var $container = $('#'+this.attributes.id);
        
        // Set grid settings based on cols
        var gridClass = "ui-grid-";
        
        // TODO: now it's no possible to rend more than 5 columns in jquery-mobile, then if attr.cols > 5 it will be ignored and 5 cols will be used
        var cols = attr.cols > 5 ? 5 : attr.cols;
        switch (Number(cols)) {
            case 1: 
            case 2: gridClass += 'a'; break;
            case 3: gridClass += 'b'; break;
            case 4: gridClass += 'c'; break;
            case 5: 
            default: gridClass += 'd'; break;
        }    
        $container.addClass(gridClass);
        
        // Adapt general category styles
        $container.css('background-color',attr.backColor);
        new BorderView(_.extend(attr.border, { el: $container}));
        new FontView(_.extend(attr.font, { el: $container}));
        
        // Fill with symbols
        var colClass = ['ui-block-a','ui-block-b','ui-block-c','ui-block-d','ui-block-e'];
        var models = this.attributes.selectables;
        var rowHeight = new Number(100 / attr.rows).toPrecision(4) + '%';
        var rowCss = {"height": rowHeight, "max-height": rowHeight};
        var lastIndex = (cols * attr.rows) - 1;
        for (var row = 0; row < attr.rows; row ++) {
            for(var col = 0; col < cols; col++) {
                var idx = (cols * row) + col;
                $container.append('<div id="grid-' + idx + '" class="'+colClass[col]+'"></div>');
                var $row = $("#grid-"+ idx);
                
                // Set row height
                $row.css(rowCss);
                
                // Add symbol
                var symbol = null;
                // Add back button
                if (idx == lastIndex) {
                    symbol = new SymbolView(_.extend(View.symbols.get('volver'), { el: $row, className: SymbolView.SYMBOL_BACK_CLASS}));
                } else if (! models[idx]) {
                    symbol = new SymbolView(_.extend(new Symbol({text:''}), { el: $row, className: SymbolView.SYMBOL_VOID_CLASS}));
                } else if (models[idx] instanceof Category) {
                    symbol = new SymbolView(_.extend(models[idx].symbol, { el: $row, className: SymbolView.SYMBOL_CATEGORY_CLASS, categoryIdx: models[idx].attributes.id}));
                } else if (models[idx] instanceof Symbol) {
                    symbol = new SymbolView(_.extend(models[idx], { el: $row}));
                }
                symbol.$el.css(rowCss);
                this.symbols.push(symbol);
            }
        }
               
        return this;
    },
    
    renderEdit: function () {
        var controls = '<button class="ui-icon-plus add" title="Añadir categoría">Añadir categoria</button>\n\
                        <button class="ui-icon-edit edit" title="Editar categoría">Editar categoría</button>';
        if (! this.attributes.immortal) {
             controls+='<button class="ui-icon-delete delete" title="Borrar categoría">categoría</button>';
        }
        View.renderControls($(controls),this.$el,12);
        for(var i=0; i<this.symbols.length; i++) {
            this.symbols[i].renderEdit();
        }
    },
    
    /**
     * Release created symbols
     */
    release: function () {
        var symbol = null;
        while (symbol = this.symbols.pop()) {
            symbol.release();
        }
    },
    
    events: {
        "click button.add": "add",
        "click button.edit": "edit",
        "click button.delete": "delete"
    },
    
    add: function () {
        app.message('TODO: Pantalla para añadir categoría','todo-dialog'); 
    },
    
    edit: function () {
        app.message('TODO: Pantalla para editar la categoría ' + this.attributes.text + '"','todo-dialog');
    },
    
    delete: function () {
        app.message('TODO: Pantalla de eliminar la categoría "' + this.attributes.text + '"','todo-dialog');
    },
});

//@ sourceURL=CategoryView.js

    