/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Class MediaView 
 * @description Provides methods to play and stop media resources
 */
var MediaView = View.extend({

    $media: null,   // Media object to manage
           
    setMedia: function (media) {
        this.$media = media;
    },
    
    getMedia: function () {
        return this.$media;
    },
    
    release: function () {
        if (this.playObject) {
            MediaView.stop(this.playObject);
        }
    }
}, {
    
    playList: new Array(),
    playing: null,
    
    /**
     * Playlist Object
     */
    PlayObject: function (uri, onPlayEnd) {
        this.id = _.uniqueId('playlist_');
        this.uri = uri;
        this.onPlayEnd = onPlayEnd;
        return this;
    },
    
    /**
     * addToPlayList
     * @param (DOMString) uri. Uri to media content.
     * @param callback Callback function on play end
     * @return (PlayListObject) 
     */
    addToPlayList: function (uri,onPlayEnd) {
        console.log('Addo to play list: '+ uri);
        var playObject = new MediaView.PlayObject(uri, onPlayEnd);
        if (MediaView.isPlaying()) {
            console.log('State is playing.');
            MediaView.playList.push(playObject);
        } else{
            console.log('State is stopped.');
            MediaView.__play(playObject);
        }
        
        return playObject;
    },
    
    /**
     * Return if there are media content playing
     */
    isPlaying: function () {
        return MediaView.playing && MediaView.playing.media;
    },
    
    /**
     * Force to play immediately
     */
    playNow: function (uri,onPlayEnd) {
        // If there is a current sound playing stop it
        if (MediaView.playing) {
            MediaView.stop(MediaView.playing);
        }
        return MediaView.__play(new MediaView.PlayListObject(uri, onPlayEnd));
    },
    
    /**
     * Stop or remove playObject from list
     */
    stop: function (playObject, onStopCallBack) {
        // Avoid errors to stop call before add symbol to play list
        if (! playObject) {
            console.log("Stop before addToPlaylist or playNow");
        } // If it's playing stop it
        else if (MediaView.playing && MediaView.playing.id == playObject.id) {
            if (! app.isCordova) {
                MediaView.playing.media.pause();
                MediaView.playing.media.src = '';
            } else {
                MediaView.playing.media.stop();
                MediaView.playing.media.release();
            }
            delete MediaView.playing.media;
            delete MediaView.playing;
        } else {
            var list = MediaView.playList;
            for (var i=0; i<list.length; i++) {
                if (list[i].id == playObject.id) {
                    list.slice(i,1);
                }
            }
        }
        if (onStopCallBack) onStopCallBack();
    },
    
    /**
     * Internal method to play
     */
    __play: function (playObject) {
        if (! MediaView.playing) {
            MediaView.playing = playObject;
            if (! app.isCordova) {
                playObject.media = new Audio(playObject.uri);
                playObject.media.addEventListener('ended',MediaView.__next);
                //playObject.media.load();
                playObject.media.play();
            } else {
                playObject.media = new Media(playObject.uri, MediaView.__next, function (error) {
                    console.log('Error [' + error.code + ' ' + error.message + '] loading media.');
                    console.log(error);
                });  
                playObject.media.play();
            }
        }
        return MediaView.playing;
    },
    
    /**
     * Internal method to play
     */
    __next: function () {
        var callBack = null;
        if (MediaView.playing && MediaView.playing.onPlayEnd) {
            callBack = MediaView.playing.onPlayEnd;
        }
        MediaView.stop(MediaView.playing);
        callBack && callBack();
        var next = MediaView.playList.pop();
        if (next) MediaView.__play(next);
    }
    
    
});

//@ sourceURL=MediaView.js