/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

var SymbolView = View.extend({
    
    defaults: {
        templatesID: ["symbol","symbolEdit"],
        className: "symbol selectable"
    },
    
    initialize: function (options) {
        this.prepare.apply(this, arguments);
        if (options) this.options = options;
        this.renderContent();
    },
    
    renderContent: function () {
        switch (this.getUseTo()) {
            case ViewUseTo.use: this.renderUse(); break;
            case ViewUseTo.edit: this.renderEdit(); break;
        }
        
    },
    
    renderUse: function () {
        var attr = this.attributes;
        var id = "symbol-" + this.attributes.id;
        var $id = "#" + id;
        var data = {id: id,
                    dataIdx: this.attributes.id,
                    categoryIdx: (this.options && this.options.categoryIdx) ? this.options.categoryIdx : '' };
        var content = '<div id="'+data.id+'" class="'+this.attributes.className+'" data-idx="' + data.dataIdx +'" ';
        if (data.categoryIdx) { 
            content+='data-categoryIdx="'+data.categoryIdx+'"';
        }
        content+='></div>';
        this.$el.append(content);
        var text = '<div class="text"><p>' + attr.text + '</p></div>';
        
        // render media content of symbol
        var renderMedia = function (obj, context) {
            var media = $('<div class="media"></div>');
            media.appendTo(obj);
            
            // if video is set, no image neither sound will be displayed
            if (attr.video && attr.video.get('uri')) {
                media.video = new VideoView(_.extend(attr.video, {el: media}));
            } else {
                // render image and hoverimage
                if (attr.image && attr.image.get('uri')) media.image = new ImageView(_.extend(attr.image, {el: media}));
                if (attr.hoverImage && attr.hoverImage.get('uri')) media.hoverImage = new ImageView(_.extend(attr.hoverImage, {el: media}));
                if (attr.sound && attr.sound.get('uri'))  media.sound = new SoundView(_.extend(attr.sound, {el: media}));
            }
            context.media = media;
        };

        var $symbol = $($id);
        switch (attr.position) {
            case SelectableSymbol.TextPosition.up: $symbol.html(text); renderMedia($symbol,this); break;
            case SelectableSymbol.TextPosition.down: renderMedia($symbol,this); $symbol.append(text); break;
        }
        
        // set created $symbol 
        this.setElement($symbol);
        
        // apply css
        this.applyNormalCSS();
        
        return this;
    },
    
    renderEdit: function () {
        var controls = '';
        if ((this.$el).hasClass(SymbolView.SYMBOL_VOID_CLASS)) {
            controls = '<button class="ui-icon-plus add" title="Añadir símbolo">Añadir</button>';
        } else {
            controls = '<button class="ui-icon-edit edit" title="Editar símbolo">Editar</button>';
            if (! this.attributes.immortal) {
                 controls+='<button class="ui-icon-delete delete" title="Borrar símbolo">Elminar</button>';
            }
        }
        View.renderControls($(controls),this.$el,13);
    },
    
    applyNormalCSS: function (removeClass) {
        obj = this.$el;
        obj.css('background-color',this.attributes.backColor);
        if (removeClass) obj.removeClass(removeClass);
        
        if ( ! this.border) { 
            this.border = new BorderView(_.extend(this.attributes.border, { el: obj}));
        } else {
            this.border.render();
        }
        
        if ( ! this.font) {
            this.font = new FontView(_.extend(this.attributes.font, {el: obj}));
        } else {
            this.font.render();
        }
        
        if ( this.media.image ) { 
            this.media.image.$img.show();
            if (this.media.hoverImage) {this.media.hoverImage.$img.hide();}
        }
    },
    
    applyFocusCSS: function (addClass) {
        obj = this.$el;
        obj.css('background-color',this.attributes.hoverBackColor);
        if (addClass) obj.removeClass(addClass);
        
        if ( ! this.hoverBorder) { 
            this.hoverBorder = new BorderView(_.extend(this.attributes.hoverBorder, { el: obj}));
        } else {
            this.hoverBorder.render();
        }
        
        if ( ! this.hoverFont) {
            this.hoverFont = new FontView(_.extend(this.attributes.hoverFont, {el: obj}));
        } else {
            this.hoverFont.render();
        }
        
        if ( this.media.hoverImage ) { 
            this.media.hoverImage.$img.show();
            if (this.media.image) {this.media.image.$img.hide();}
        }
    },
    
    /**
     * Release media resources associated to symbol
     */
    release: function () {
        if (this.media) {
            if (this.media.sound) {
                this.media.sound.release();
            }
            if (this.media.video) {
                this.media.video.release();
            }
        }
    },
    
    /**
     * Events handlers
     */    
    events: {
        "focalize": "focalize",
        "unfocalize": "unfocalize",
        "play": "play",
        "click button.add": "add",
        "click button.edit": "edit",
        "click button.delete": "delete",
    },
    
    add: function () {
        app.message('TODO: Pantalla para añadir símbolos','todo-dialog'); 
    },
    
    edit: function () {
        app.message('TODO: Pantalla para editar el símbolo ' + this.attributes.text + '"','todo-dialog');
    },
    
    delete: function () {
        app.message('TODO: Pantalla de eliminar el símbolo "' + this.attributes.text + '"','todo-dialog');
    },
      
    focalize: function (event, onEnded) {
    
        // Void symbol's can't be selecteds
        if (this.$el.hasClass(SymbolView.SYMBOL_VOID_CLASS)) {
            event.stopImmediatePropagation();
        } else {
            if (this.attributes.sound.attributes.autoPlay) {
                this.play(event);
            }
            else {
                this.applyFocusCSS();
            }
        }
    },
    
    unfocalize: function (event) {

        // Void symbol's can't be selecteds
        if (this.$el.hasClass(SymbolView.SYMBOL_VOID_CLASS)) {
            event.stopImmediatePropagation();
        } else {
            // Stop event 
            this.stop(event);
        }
    },
    
    /**
     * Play media content
     */
    play: function (event, onEnded) {
        //console.log("Symbol Play: " + this.id);
        this.applyFocusCSS();
        if (this.media.sound) {
            this.media.sound.play(onEnded);
        } else if (onEnded) {
            onEnded();
        }
    },
    
    /**
     * Stop media content
     */
    stop: function (event, onReadyAgain) {
        console.log("Symbol Stop: " + onReadyAgain);
        this.applyNormalCSS();
        if (this.media.sound) {
            this.media.sound.stop(onReadyAgain);
        } else if (onReadyAgain) {
            onReadyAgain();
        }
    },
    
}, {
    SYMBOL_CATEGORY_CLASS: 'category-symbol',
    SYMBOL_VOID_CLASS: 'void',
    SYMBOL_BACK_CLASS: 'back-symbol',
    SYMBOL_NEXT_CLASS: 'next-symbol'
});

//@ sourceURL=SymbolView.js
