/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

var ViewUseTo = {
    use: 0,
    edit: 1
};

var View = Backbone.View.extend({
    
    prepare: function (arguments) {
        var defaults = _.defaults(this.defaults,View.defaults);
        if (arguments) defaults.useTo = arguments.useTo || defaults.useTo;
        this.attributes = _.defaults(this.attributes || {}, defaults);
        if (arguments && arguments.className) {
            this.attributes.className = this.attributes.className ? this.attributes.className + ' ' + arguments.className : arguments.className;
        }
        this.dbURI = arguments.dbURI || bm.dbURI;   // If not dbURI received, dbURI from global board manager is getted
        this.board = bm.board;                      // Link to active board
    },
    
    render: function() {
        this.$el.html(this.getRenderedString());
        return this;
    },
    
    renderByData: function (data) {
        this.$el.html(data);
        return this;
    },
    
    getRenderedString: function (data) {
        data = _.defaults(data || {},this.attributes);
        return _.template(View.getTemplate(this.getTemplateName()),data);
    },
    
    getTemplateName: function () {
        return this.attributes.templatesID[this.getUseTo()];
    },
    
    setUseTo: function(useTo) {
        this.attributes.useTo = useTo;
    },
    
    getUseTo: function() {
        return this.attributes.useTo;
    }
}, {
    
    categories: {},               // Collection of categories
    symbols: {},                  // Collection of symbols
    
    defaults: {                   // Default attibutes
        useTo: ViewUseTo.use,     // default index
        templatesID: ["",""]      // Templates ids must be overriden by each subclass
    },
    
    templates: {},
    
    getTemplate: function(id) {
        return View.templates[id];
    },   
    
    setTemplate: function (id, template) {
        View.templates[id] = template;
    },
    
    renderControls: function (controls, parent, zIndex) {
        var $parent = $(parent);
        var controlDiv = $('<div class="edit-controls"></div>');
        // Add default class to buttons
        controls.addClass('ui-btn ui-shadow ui-corner-all ui-btn-icon-notext ui-btn-inline');
        controlDiv.append(controls);
        $parent.append(controlDiv);
        controlDiv.offset($parent.offset());
        controlDiv.width($parent.width());
        //controlDiv.height($parent.height());
        controlDiv.attr('z-index',zIndex);
    }
});

//@ sourceURL=View.js