/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

// Override toJSON BackBone method 
Backbone.Model.prototype.toJSON = function () {
    var jsonArray = {};
    var attr = this.attributes;
    var keys = null;
    if (this.exportAttributes) {
        keys = this.exportAttributes();
    } else {
        keys = _.keys(attr);
    }
    for (var i=0; i < keys.length; i++) {
        if (attr[keys[i]] && attr[keys[i]].toJSON) {
            jsonArray[keys[i]] = attr[keys[i]].toJSON();
        } else {
            jsonArray[keys[i]] = attr[keys[i]];
        }
    }
    return jsonArray;
};

var app = {
    
    config: {
        filePath: 'config.json',                                // Local config file
        server: 'http://192.168.0.192/MultiSAAC/',              // Server
        serverActionsRoot:  'public/',                          // Server actions root path
        serverDBRoot: 'data/users/',                            // Server database root path
        localDBRoot: 'multisaac/',                              // Local database root path
        debug: true                                             // Debug mode
    },
    
    pageIndex: 'index.html',
    basePath: document.URL.replace(/index.html.*$/,''),     // base path of root application
    isMoble: isMobile.any,                                  // Shorthand to isMobile.any
    isCordova: window.isCordovaApp,                         // Tell if running instance is inside cordova webview 
    
    router: new $.mobile.Router([]),                        // Router manager
                    
    
    // Application Constructor
    initialize: function() {
        app.isCordova = sessionStorage.getItem('isCordovaApp');
        // Bind event
        app.bindEvents();
    },
    
    /**
     * Load configuration and startApplication
     */
    load: function () {
        console.log("App load ...");
        // Force to reload to index.html to reset app on reload action (reload index.html#hash -> index.html)
        if (window.location != (app.basePath + app.pageIndex)) {
            window.location = app.basePath + app.pageIndex;
        } else { 
            
            var fail = function () {
                            console.log('Starting error!!');
                            app.hideLoadingMessage();
                            app.alertMessage('Ha ocurrido un error inesperado iniciando la aplicación.<br>\n\
                                              Por favor, intente reinstalar la aplicación o póngase en contacto con nosotros.');
                        };
            app.loadConfig()
                //.done(app.loadLanguajes)
                //.done(app.loadLibraries)
                //.done(app.loadTemplates)
                .done(function () {
                    app.initLocalDBDir()
                    .done(function () {
                        console.log('Initializing objects ...');
                        app.serverActionPath = app.config.server + app.config.serverActionsRoot;
                        // Initialize Profile Manager
                        pm.initialize();
                        // Initialize Board Manager
                        bm.initialize();
                        app.start();
                    })
                    .fail (fail);
                })
                .fail (fail);
        }
    },
    
    // Bind Event Listeners
    //
    // Bind any events that are required on startup. Common events are:
    // 'load', 'deviceready', 'offline', and 'online'.
    bindEvents: function() {
        console.log("Binding events...");
        
        // On debug mode, always trace jquery connection
        if (app.debug) {
            console.log("Debug mode enabled...");
            $( document ).ajaxError(function( event, request, settings, exception) {
                console.log("Exception [" + exception + "] requesting page [" + settings.url + "]");
            });
        }
        
        if (app.isCordova) {
            document.addEventListener('deviceready', app.startCordova, false);
        } else if (app.isMobile) {
            app.startMobile();
        } else {
            app.startWeb();
        }
    },
    
    startCordova: function () {
        console.log("Cordova enabled...");
        app.isCordova = true;
        app.startMobile();
    },
    
    startMobile: function () {
        console.log("Mobile enabled...");
        app.isMobile = true;
        $(document).ready(app.load());
    },
    
    startWeb: function () {
        console.log("Web enabled ...");
        $(document).ready(app.load());
    },
    
    // deviceready and DOMContentLoaded Event Handler
    //
    // The scope of 'this' is the event. In order to call the 'receivedEvent'
    // function, we must explicity call 'app.receivedEvent(...);'
    start: function () {
        try {
            console.log("App start  ...");
            var defaultUser = pm.getDefaultUser();
            if (! defaultUser) {
                console.log("No default user.");
                app.mainAction();
            } else {
                bm.loadBoardAction(defaultUser);
            }
        }
        catch(excepton) {
            console.log("Unexpected error: " + excepton);
            app.alertMessage('Ha ocurrido un error inesperado. Puede que la aplicación no funcione adecuadamente.');
        }
    },
    
    /**
     * CloseBoard action
     * @returns {undefined}
     */
    closeBoardAction: function ()  {
        app.mainAction();
    },
    
    /**
     * Main action
     * @returns {undefined}
     */
    mainAction: function () {
        pm.showUserListAction(function (user) {
            bm.loadBoardAction(user);
        });
    },
    
    /**
     * Indicates if device is connected 
     * @param {Boolean} broadBandOnly Optional default=true. If true only ETHERNET and WIFI connections return true. Otherwise any connection will return true 
     * @returns {Boolean} 
     */
    isConnected: function (broadBandOnly) {
        if (broadBandOnly === false) {
            return navigator.connection.type != Connection.NONE;
        } else {
            return navigator.connection.type == (Connection.WIFI || Connection.ETHERNET);
        }
    },
    
    /**
     * Load page from server and append to parent
     * @param {String} page
     * @param {DOMElement} parent
     * @param {PlainObject or String} param Optional params to the server
     * @returns {Promise} Promise Object
     */
    loadServerPage: function (page, parent, params) {
        return $.get(app.serverActionPath + page, params).done(function (data) {
            $(parent).html(data);
        });
    },
    
    /**
     * Change page
     * @param {String} page Relative url to load
     * @param {Object} options Optional options pagecontainer widget
     * @returns {unresolved}
     */
    loadLocalPage: function (page, options) {
        var defaults = {
            'transition': 'slidedown'
        };
        console.log('Changig to page: ' + page);
        return $(':mobile-pagecontainer').pagecontainer('change', app.basePath + page, $.extend({}, defaults, options));
    },
    
    /**
     * Local request
     * @param {String} page
     * @param {PlainObject or String} params
     * @returns {Promise} Promise Object
     */
    localRequest: function (page, params) {
        return $.get(app.basePath + page, params);
    },
    
    /**
     * Load script from local
     * @param {String} page
     * @param {PlainObject or String} params
     * @returns {Promise} Promise Object
     */
    loadScript: function (page, options) {
        var url = app.basePath + page; 
        var defaultOptions = {
                cache: false,
                async: false,       // Script loader is synchronous to respect dependencies
                url: url
            };
        // Allow user to set any option except for dataType, cache, and url
        // Allow user to set any option 
        var opt = _.clone(options) || {};
        opt = _.defaults(opt, defaultOptions );
        var promise = $.ajax(opt)
                        .done(function (data) {
                            $.globalEval(data);
                        })
                        .fail(function(jqXHR, textStatus, errorThrown) {
                            console.log('Error ['+ errorThrown +'] loading script ['+ url +']')
                        });
        return promise;
    },
        
    /**
     * Send a request to the server
     * @param {String} page
     * @param {PlainObject or String} params
     * @returns {Promise} Promise Object
     */
    serverRequest: function (page, params) {
        return $.post(app.serverActionPath + page, params);
    },
    
    /**
     * Show a message
     * @param {String} message
     * @param (String) css Optional class to add to the content
     * @returns {Popup element}
     */
    message: function (message, css, timeout) {
        css = css || '';
        var m = $('<div data-role="popup" id="message" class="ui-content ui-popup ui-body-a ui-overlay-shadow ui-corner-all ' + css +'"><p>'+message+'</p></div>');
        m.popup();
        m.popup('open');
        if (timeout) {
            setTimeout( function () {$(m).popup('close');},timeout);
        }
        return m;
    },
    
    /**
     * Shows a informative message
     * @param {String} message    
     * @param {int} timeout Optional. Default 3000
     * @param {String} css Optional. Additional CSS class to the message
     * @returns {undefined}
     */
    infoMessage: function(message, timeout, css) {
        app.message(message,'ui-icon-info ui-btn-icon-left ' + css,timeout || 3000);
    },
    
    /**
     * Shows a alert message
     * @param {String} message    
     * @param {int} timeout Optional. Default 3000
     * @param {String} css Optional. Additional CSS class to the message
     * @returns {undefined}
     */
    alertMessage: function (message, timeout, css) {
        app.message(message,'ui-icon-alert ui-btn-icon-left ' + css,timeout || 3000);
    },
    
    /**
     * Shows loading message
     * @param {String} msg Optional. Loading message
     * @returns {undefined}
     */
    loadingMessage: function (msg) {
        $.mobile.loading( 'show', {
                 text: msg,
                 textVisible: msg,
                 textonly: false,
                 html: ""
            });
    },
    
    /**
     * Hide loading message
     * @returns {undefined}
     */
    hideLoadingMessage: function () {
        $.mobile.loading ( 'hide' );
    },
        
    /**
     * Load configuration
     * @returns {Promise} Promise Object
     */
    loadConfig: function () {
        var deferred = $.Deferred();
        app.loadingMessage('Cargando configuración');
        console.log('Loading configuration ...');
        app.localRequest(app.config.filePath)
            .done(function (data) {
                app.config = $.extend({}, app.config, data);
                console.log('Configuration loaded.');
                deferred.resolve(data);
            })
            .fail (function (jqXHR, textStatus, errorThrown) {
                console.log("Error [" + errorThrown + "] loading config file.");
                deferred.reject(jqXHR, textStatus, errorThrown);
            });
        return deferred.promise();
         
    },
    
    /**
     * Load languaje files
     * @returns {Promise} Promise Object
     */
    loadLanguajes: function () {
        return app.loadScript(app.config.languaje.lib)
           .done(function () {
                var langOpt = { 
                   ns: { 
                       namespaces: ['multisaac'],   //MultiSAAC.config.languaje.namespaces, 
                       defaultNs: 'multisaac'       //MultiSAAC.config.languaje.defaultNamespace
                   },
                   fallbackLng: false,
                   debug: true, //MultiSAAC.config.debug,
                   getAsync: false,
                   useLocalStorage: false,
                   localStorageExpirationTime: 86400000 
                };// in ms, default 1 week
                 //fallbackLng: MultiSAAC.config.languaje.defaultLang,
                $.i18n.init(langOpt); // Initialize lang files
                //$.i18n.loadNamespaces(['multisaac','foo']) ;
            })
            .fail(function (jqXHR, textStatus, errorThrown) {
                console.log("Error [" + errorThrown + "] loading languaje files.");
            });
    },

    /**
     * Load javascript libraries
     * @returns {Promise} Promise Object
     */
    loadLibraries: function () {
        var deferred = $.Deferred();
        var promises = new Array();
        app.loadingMessage('Cargando librerias');
        console.log('Loading libraries ...');
        _.each(app.config.libs.load,function (o) {
            promises.push(app.loadScript(app.config.libs.path + o.src));
        });
        $.when(promises).done(function () {
            deferred.resolve();
        }).fail(function (jqXHR, textStatus, errorThrown) {
            console.log('Error ['+ errorThrown +'] loading libraries');
            deferred.reject();
        });
        return deferred.promise();
    },
        
    /**
     * Load Templates
     * @returns {Promise} Promise Object
     */
    loadTemplates: function () {
        var promises = new Array();
        app.loadingMessage('Cargando templates');
        console.log('Loading templates ...');
        _.each(app.config.templates.load,function (o) {
            promises.push(function (id,src) {
                return app.localRequest(app.config.templates.path + src,{dataType: 'text', cache: true})
                                .done(function(data) {
                                    View.setTemplate(id,data);
                                });
                }(o.id,o.src));
        });
        return $.when(promises).fail(function (jqXHR, textStatus, errorThrown) {
            console.log('Error ['+ errorThrown +'] loading templates');
            console.log(errorThrown);
        });
    },
   
    /**
     * Grant access to filesystem
     * @returns {Promise} Promise object
     */
    initLocalFileSystem: function () {
        var def = $.Deferred();
        console.log('Initializing file system ...');
        if (app.FS) {
            console.log('File system already started.');
            def.resolve(app.FS);
        } else {
            console.log('Requesting for file system ...');
            // request the persistent file system
            window.requestFileSystem(LocalFileSystem.PERSISTENT, 1024 * 1024 * 50, function (fs) {
                console.log('File System Getted. Root ['+fs.root.name+']');
                app.FS = fs;
                def.resolve(fs);
            }, function (error) {
                console.log("Error [" + error.code + "] getting filesystem.");
                def.reject(error);
            });
        }
        return def.promise();
    },
    
    /**
     * Get db directory (on non mobile devices, resolve(null) will fired)
     * @returns {Promise} Promise object
     */
    initLocalDBDir: function () {
        var deferred = $.Deferred();
        console.log('Initializing local DB ...');
        app.loadingMessage('Conectando la base de datos ...');
        if (! app.isMobile) {
            console.log('Web version, no local db connection.');
            deferred.resolve(true);
        } else if (app.LocalDBDir) {
            console.log('Connection to local db already started.');
            deferred.resolve(app.LocalDBDir);
        } else {
            console.log('Requesting file system.');
            app.initLocalFileSystem()
            .done(function (fs) {
                var dirPath = app.config.localDBRoot;
                console.log("FileSystem obtained, getting directory ["+dirPath+"] ...");
                var succes = function (dir) {
                    console.log("Directory getted. LocalPath ["+dir.fullPath+"]");
                    app.LocalDBDir = dir;
                    deferred.resolve(dir);
                };
                fs.root.getDirectory(dirPath, {create: true}, succes, function (error) {
                    if (error.code == 12) {
                        console.log("Directory creation fail. Getting existing directory.");
                        fs.root.getDirectory(dirPath, { create: false, exclusive: false }, succes, function (error) {
                            console.log("Error [" + error.code + "] getting directory ["+ dirPath +"]")
                            deferred.reject(error);
                        });
                    } else {
                        console.log("Error [" + error.code + "] getting directory ["+ dirPath +"]")
                        deferred.reject(error);
                    }
                });
            })
            .fail(function (error) {
                console.log('Error ['+error.code+'] initializing File System.');
                deferred.reject(error);
            });
        }
        return deferred.promise();
    },
    
    /**
     * Get a Directory Entry for user
     * @param {String} userKey
     * @returns {Promise} Promise object
     */
    getUserDBDir: function (userKey) {
        var deferred = $.Deferred();

        console.log("Getting user dir for user ["+userKey+"]");
        var succes = function (dir) {
            console.log("Directory getted. LocalPath ["+dir.fullPath+"]");
            deferred.resolve(dir);
        };
        
        var baseDir = app.getLocalDBDir();
        baseDir.getDirectory(userKey, { create: true, exclusive: false }, succes, function (error) {
            if (error.code == 12) {
                baseDir.getDirectory(userKey, { create: false, exclusive: false }, succes, function (error) {
                    console.log("Error [" + error.code + "] getting directory ["+ userKey +"]")
                    deferred.reject(error);
                });
            } else {
                console.log("Error [" + error.code + "] getting directory ["+ userKey +"]")
                deferred.reject(error);
            }
        });
        
        return deferred.promise();                
    },
    
    /**
     * Read file from local filesystem uri
     * @param {type} uri
     * @param {type} readType
     * @returns {undefined}
     */
    getLocalFileFromURI: function (uri, readType) {
        var deferred = $.Deferred();
        
        var fail = function (fileError) {
            var error = 'Error ['+fileError.code+'] getting ['+uri+']';
            console.log(error);
            deferred.reject(error);
        };
        
        window.resolveLocalFileSystemURI(uri,
            function (fileEntry) {
                fileEntry.file(
                    function (f) {
                        //console.log('File Entry getted, creating reader ...');
                        var reader = new FileReader();
                        reader.onloadend = function(evt) {
                            console.log('File ['+uri+'] read end ...');
                            deferred.resolve(reader.result);
                        };
                        reader.onerror = fail;
                        switch (readType) {
                            case 'array': reader.readAsArrayBuffer(f); break;
                            case 'binary': reader.readAsBinaryString(f); break;
                            case 'url': reader.readAsDataURL(f); break;
                            case 'text': 
                            default: reader.readAsText(f); break;
                        }
                    }, 
                fail);
            }
        ,fail);
            
        return deferred.promise();
    },
    
    /**
     * Returns Local File System Object
     * @returns {FileSystem}
     */
    getFS: function () {
        return app.FS;
    },
    
    /**
     * Returns Local File System Dir
     * @returns {DirectoryEntry}
     */
    getLocalDBDir: function () {
        return app.LocalDBDir;
    }
};

//@ sourceURL=index.js