<?php

/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Media\Controller;

use Zend\Mvc\Controller\AbstractActionController;
use Application\Model\Application;
use Zend\View\Model\JsonModel;
use Media\Form\MediaForm;
use Zend\Filter\File\RenameUpload;

class MediaController extends AbstractActionController
{
    
    /**
     * Default action
     * @return string
     */
    public function indexAction()
    {
        return new JsonModel(array('result' => 'INVALID ACTION'));
    }

    /**
     * Create media
     * @return type
     */
    public function putAction()
    {
        $key = $this->params()->fromRoute('key', 0);
        if (!$key) {
            return new JsonModel(array('result' => 'NO KEY RECEIVED'));
        }
        
        $request = $this->getRequest();
        if (! $request->isPost()) {
            return new JsonModel(array('result' => 'INVALID REQUEST'));
        } else {
            $form = new MediaForm();
            // Make certain to merge the files info!
            $post = array_merge_recursive(
                $request->getPost()->toArray(),
                $request->getFiles()->toArray()
            );

            $form->setData($post);
            if ($form->isValid()) {
                $data = $form->getData();
                if ($data['file']['error']) {
                    return new JsonModel(array('result' => 'ERROR UPLOADING FILE', 'error' => $data['file']['error']));
                } else if (! $this->validateMime($data['file']['type'])) {
                    return new JsonModel(array('result' => 'INVALID MIME TYPE'));
                } else if (! $this->validateSize($data['file']['size'])) {
                    return new JsonModel(array('result' => 'INVALID FILE SIZE'));
                } else {
                    $data_dir = Application::getMediaDir($key);
                    if (!file_exists($data_dir)) {
                        return new JsonModel(array('result' => 'INVALID KEY'));
                    } else {
                        $filename = $data_dir.preg_replace('/[^a-z0-9_\.-]/ui', '-', $data['file']['name']);
                        Application::getLogger()->info($filename);
                        $filter = new RenameUpload(array(
                                                    "target"    => $filename,
                                                    "randomize" => true,
                                                ));
                        try {
                            $file = $filter->filter($data['file']);
                            return new JsonModel(array('result' => 'SAVED', 'name' => basename($file['tmp_name'])));
                        } catch (Exception $ex) {
                            return new JsonModel(array('result' => 'SAVED'));
                        }
                    }
                }
            }
        }
    }

    private function validateMime($mime) {
        return true;
    }
    
    private function validateSize($size) {
        return true;
    }
    /**
     * Get media
     * @return string
     */
    public function getAction()
    {
        $key = $this->params()->fromRoute('key', 0);
        if (!$key) {
            return new JsonModel(array('result' => 'NO KEY RECEIVED'));
        }
        $request = $this->getRequest();
        $file = $this->params()->fromRoute('file', '');
        $file = $file ? $file : $request->getPost('file');
        if (! $file) {
            return new JsonModel(array('result' => 'NO FILE REQUEST'));
        }

        $data_dir = Application::getMediaDir($key);
        $filename = $data_dir.$file;

        if (!file_exists($filename)) {
            return new JsonModel(array('result' => 'FILE NOT EXISTS'));
        } else {
            $fileContents = file_get_contents($filename);
            
            $response = $this->getResponse();
            $response->setContent($fileContents);
            
            $headers = $response->getHeaders();
            $headers->clearHeaders()
                ->addHeaderLine('Content-Type', $this->getMime($filename))
                ->addHeaderLine('Content-Disposition', 'attachment; filename="' . basename($filename) . '"')
                ->addHeaderLine('Content-Length', filesize ($filename));
            return $this->response;
        }
    }
    
    private function getMime($filename) {
        $data = $this->get_file_mime_type($filename,true);
        return $data['mime_type'];
    }
    
    /**
     * Delete media
     * @return string
     */
    public function deleteAction()
    {
        $key = $this->params()->fromRoute('key', 0);
        if (!$key) {
            new JsonModel(array('result' => 'NO KEY RECEIVED'));
        }
        
        $data_dir = Application::getDataDir($key);
        $filename = $data_dir.'board.json';
        Application::getLogger()->info($filename);
        if (!file_exists($filename)) {
            Application::getLogger()->err('File not found');
            return new JsonModel(array('result' => 'FILE NOT EXISTS'));
        } else {
            // get image content
            $response = $this->getResponse();
            $content = file_get_contents($filename);
            $response->setContent($content);
            $response
                ->getHeaders()
                ->addHeaderLine('Content-Type', 'application/json')
                ->addHeaderLine('Content-Length', mb_strlen($content));

            return $response;
        }
    }
    
    /**
     * This function is get from Chris Jean http://chrisjean.com/2009/02/14/generating-mime-type-in-php-is-not-magic/
     */ 
    private function get_file_mime_type( $filename, $debug = false ) {
        if ( function_exists( 'finfo_open' ) && function_exists( 'finfo_file' ) && function_exists( 'finfo_close' ) ) {
            $fileinfo = finfo_open( FILEINFO_MIME );
            $mime_type = finfo_file( $fileinfo, $filename );
            finfo_close( $fileinfo );

            if ( ! empty( $mime_type ) ) {
                if ( true === $debug )
                    return array( 'mime_type' => $mime_type, 'method' => 'fileinfo' );
                return $mime_type;
            }
        }
        
        if ( function_exists( 'mime_content_type' ) ) {
            $mime_type = mime_content_type( $filename );

            if ( ! empty( $mime_type ) ) {
                if ( true === $debug )
                    return array( 'mime_type' => $mime_type, 'method' => 'mime_content_type' );
                return $mime_type;
            }
        }

        $mime_types = array(
            'ai'      => 'application/postscript',
            'aif'     => 'audio/x-aiff',
            'aifc'    => 'audio/x-aiff',
            'aiff'    => 'audio/x-aiff',
            'asc'     => 'text/plain',
            'asf'     => 'video/x-ms-asf',
            'asx'     => 'video/x-ms-asf',
            'au'      => 'audio/basic',
            'avi'     => 'video/x-msvideo',
            'bcpio'   => 'application/x-bcpio',
            'bin'     => 'application/octet-stream',
            'bmp'     => 'image/bmp',
            'bz2'     => 'application/x-bzip2',
            'cdf'     => 'application/x-netcdf',
            'chrt'    => 'application/x-kchart',
            'class'   => 'application/octet-stream',
            'cpio'    => 'application/x-cpio',
            'cpt'     => 'application/mac-compactpro',
            'csh'     => 'application/x-csh',
            'css'     => 'text/css',
            'dcr'     => 'application/x-director',
            'dir'     => 'application/x-director',
            'djv'     => 'image/vnd.djvu',
            'djvu'    => 'image/vnd.djvu',
            'dll'     => 'application/octet-stream',
            'dms'     => 'application/octet-stream',
            'doc'     => 'application/msword',
            'dvi'     => 'application/x-dvi',
            'dxr'     => 'application/x-director',
            'eps'     => 'application/postscript',
            'etx'     => 'text/x-setext',
            'exe'     => 'application/octet-stream',
            'ez'      => 'application/andrew-inset',
            'flv'     => 'video/x-flv',
            'gif'     => 'image/gif',
            'gtar'    => 'application/x-gtar',
            'gz'      => 'application/x-gzip',
            'hdf'     => 'application/x-hdf',
            'hqx'     => 'application/mac-binhex40',
            'htm'     => 'text/html',
            'html'    => 'text/html',
            'ice'     => 'x-conference/x-cooltalk',
            'ief'     => 'image/ief',
            'iges'    => 'model/iges',
            'igs'     => 'model/iges',
            'img'     => 'application/octet-stream',
            'iso'     => 'application/octet-stream',
            'jad'     => 'text/vnd.sun.j2me.app-descriptor',
            'jar'     => 'application/x-java-archive',
            'jnlp'    => 'application/x-java-jnlp-file',
            'jpe'     => 'image/jpeg',
            'jpeg'    => 'image/jpeg',
            'jpg'     => 'image/jpeg',
            'js'      => 'application/x-javascript',
            'kar'     => 'audio/midi',
            'kil'     => 'application/x-killustrator',
            'kpr'     => 'application/x-kpresenter',
            'kpt'     => 'application/x-kpresenter',
            'ksp'     => 'application/x-kspread',
            'kwd'     => 'application/x-kword',
            'kwt'     => 'application/x-kword',
            'latex'   => 'application/x-latex',
            'lha'     => 'application/octet-stream',
            'lzh'     => 'application/octet-stream',
            'm3u'     => 'audio/x-mpegurl',
            'man'     => 'application/x-troff-man',
            'me'      => 'application/x-troff-me',
            'mesh'    => 'model/mesh',
            'mid'     => 'audio/midi',
            'midi'    => 'audio/midi',
            'mif'     => 'application/vnd.mif',
            'mov'     => 'video/quicktime',
            'movie'   => 'video/x-sgi-movie',
            'mp2'     => 'audio/mpeg',
            'mp3'     => 'audio/mpeg',
            'mpe'     => 'video/mpeg',
            'mpeg'    => 'video/mpeg',
            'mpg'     => 'video/mpeg',
            'mpga'    => 'audio/mpeg',
            'ms'      => 'application/x-troff-ms',
            'msh'     => 'model/mesh',
            'mxu'     => 'video/vnd.mpegurl',
            'nc'      => 'application/x-netcdf',
            'odb'     => 'application/vnd.oasis.opendocument.database',
            'odc'     => 'application/vnd.oasis.opendocument.chart',
            'odf'     => 'application/vnd.oasis.opendocument.formula',
            'odg'     => 'application/vnd.oasis.opendocument.graphics',
            'odi'     => 'application/vnd.oasis.opendocument.image',
            'odm'     => 'application/vnd.oasis.opendocument.text-master',
            'odp'     => 'application/vnd.oasis.opendocument.presentation',
            'ods'     => 'application/vnd.oasis.opendocument.spreadsheet',
            'odt'     => 'application/vnd.oasis.opendocument.text',
            'ogg'     => 'application/ogg',
            'otg'     => 'application/vnd.oasis.opendocument.graphics-template',
            'oth'     => 'application/vnd.oasis.opendocument.text-web',
            'otp'     => 'application/vnd.oasis.opendocument.presentation-template',
            'ots'     => 'application/vnd.oasis.opendocument.spreadsheet-template',
            'ott'     => 'application/vnd.oasis.opendocument.text-template',
            'pbm'     => 'image/x-portable-bitmap',
            'pdb'     => 'chemical/x-pdb',
            'pdf'     => 'application/pdf',
            'pgm'     => 'image/x-portable-graymap',
            'pgn'     => 'application/x-chess-pgn',
            'png'     => 'image/png',
            'pnm'     => 'image/x-portable-anymap',
            'ppm'     => 'image/x-portable-pixmap',
            'ppt'     => 'application/vnd.ms-powerpoint',
            'ps'      => 'application/postscript',
            'qt'      => 'video/quicktime',
            'ra'      => 'audio/x-realaudio',
            'ram'     => 'audio/x-pn-realaudio',
            'ras'     => 'image/x-cmu-raster',
            'rgb'     => 'image/x-rgb',
            'rm'      => 'audio/x-pn-realaudio',
            'roff'    => 'application/x-troff',
            'rpm'     => 'application/x-rpm',
            'rtf'     => 'text/rtf',
            'rtx'     => 'text/richtext',
            'sgm'     => 'text/sgml',
            'sgml'    => 'text/sgml',
            'sh'      => 'application/x-sh',
            'shar'    => 'application/x-shar',
            'silo'    => 'model/mesh',
            'sis'     => 'application/vnd.symbian.install',
            'sit'     => 'application/x-stuffit',
            'skd'     => 'application/x-koan',
            'skm'     => 'application/x-koan',
            'skp'     => 'application/x-koan',
            'skt'     => 'application/x-koan',
            'smi'     => 'application/smil',
            'smil'    => 'application/smil',
            'snd'     => 'audio/basic',
            'so'      => 'application/octet-stream',
            'spl'     => 'application/x-futuresplash',
            'src'     => 'application/x-wais-source',
            'stc'     => 'application/vnd.sun.xml.calc.template',
            'std'     => 'application/vnd.sun.xml.draw.template',
            'sti'     => 'application/vnd.sun.xml.impress.template',
            'stw'     => 'application/vnd.sun.xml.writer.template',
            'sv4cpio' => 'application/x-sv4cpio',
            'sv4crc'  => 'application/x-sv4crc',
            'swf'     => 'application/x-shockwave-flash',
            'sxc'     => 'application/vnd.sun.xml.calc',
            'sxd'     => 'application/vnd.sun.xml.draw',
            'sxg'     => 'application/vnd.sun.xml.writer.global',
            'sxi'     => 'application/vnd.sun.xml.impress',
            'sxm'     => 'application/vnd.sun.xml.math',
            'sxw'     => 'application/vnd.sun.xml.writer',
            't'       => 'application/x-troff',
            'tar'     => 'application/x-tar',
            'tcl'     => 'application/x-tcl',
            'tex'     => 'application/x-tex',
            'texi'    => 'application/x-texinfo',
            'texinfo' => 'application/x-texinfo',
            'tgz'     => 'application/x-gzip',
            'tif'     => 'image/tiff',
            'tiff'    => 'image/tiff',
            'torrent' => 'application/x-bittorrent',
            'tr'      => 'application/x-troff',
            'tsv'     => 'text/tab-separated-values',
            'txt'     => 'text/plain',
            'ustar'   => 'application/x-ustar',
            'vcd'     => 'application/x-cdlink',
            'vrml'    => 'model/vrml',
            'wav'     => 'audio/x-wav',
            'wax'     => 'audio/x-ms-wax',
            'wbmp'    => 'image/vnd.wap.wbmp',
            'wbxml'   => 'application/vnd.wap.wbxml',
            'wm'      => 'video/x-ms-wm',
            'wma'     => 'audio/x-ms-wma',
            'wml'     => 'text/vnd.wap.wml',
            'wmlc'    => 'application/vnd.wap.wmlc',
            'wmls'    => 'text/vnd.wap.wmlscript',
            'wmlsc'   => 'application/vnd.wap.wmlscriptc',
            'wmv'     => 'video/x-ms-wmv',
            'wmx'     => 'video/x-ms-wmx',
            'wrl'     => 'model/vrml',
            'wvx'     => 'video/x-ms-wvx',
            'xbm'     => 'image/x-xbitmap',
            'xht'     => 'application/xhtml+xml',
            'xhtml'   => 'application/xhtml+xml',
            'xls'     => 'application/vnd.ms-excel',
            'xml'     => 'text/xml',
            'xpm'     => 'image/x-xpixmap',
            'xsl'     => 'text/xml',
            'xwd'     => 'image/x-xwindowdump',
            'xyz'     => 'chemical/x-xyz',
            'zip'     => 'application/zip'
        );

        $ext = strtolower( array_pop( explode( '.', $filename ) ) );

        if ( ! empty( $mime_types[$ext] ) ) {
            if ( true === $debug )
                    return array( 'mime_type' => $mime_types[$ext], 'method' => 'from_array' );
            return $mime_types[$ext];
        }

        if ( true === $debug )
            return array( 'mime_type' => 'application/octet-stream', 'method' => 'last_resort' );
        return 'application/octet-stream';
    }
}
