<?php

/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Profile\Model;

use Zend\InputFilter\InputFilter;
use Zend\InputFilter\InputFilterInterface;
 
class Profile
{
    public $id;
    public $mail;
    public $password;
    public $key;
    public $name;
    public $lastname;
    
    protected $inputFilter;
    protected $adapter;

    public function exchangeArray($data)
    {
        $this->id   = (!empty($data['id'])) ? $data['id'] : null;
        $this->mail = (!empty($data['mail'])) ? $data['mail'] : null;
        $this->password = (!empty($data['password'])) ? $data['password'] : null;
        $this->key  = (!empty($data['key'])) ? $data['key'] : null;
        $this->name = (!empty($data['name'])) ? $data['name'] : null;
        $this->lastname = (!empty($data['lastname'])) ? $data['lastname'] : null;
    }
    
    
    public function getArrayCopy()
    {
        return get_object_vars($this);
    }
    
   
    public function setInputFilter(InputFilterInterface $inputFilter)
    {
        throw new \Exception("Not used");
    }

    public function getInputFilter()
    {
        if (!$this->inputFilter) {
            $inputFilter = new InputFilter();

            $inputFilter->add(array(
                'name'     => 'id',
                'required' => true,
                'filters'  => array(
                    array('name' => 'Int'),
                ),
            ));

            $inputFilter->add(array(
                'name'     => 'name',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                    array(
                        'name'    => 'StringLength',
                        'options' => array(
                            'encoding' => 'UTF-8',
                            'min'      => 1,
                            'max'      => 75,
                        ),
                    ),
                ),
            ));

            $inputFilter->add(array(
                'name'     => 'lastname',
                'required' => false,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                    array(
                        'name'    => 'StringLength',
                        'options' => array(
                            'encoding' => 'UTF-8',
                            'min'      => 1,
                            'max'      => 100,
                        ),
                    ),
                ),
            ));

            $inputFilter->add(array(
                'name'     => 'mail',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                   array(
                       'name'    => 'StringLength',
                       'options' => array(
                          'encoding' => 'UTF-8',
                          'min'      => 1,
                          'max'      => 100,
                       ),
                   ),
                   array (
                      'name'    => 'EmailAddress',
                   ),
                   array (
                       'name'   => 'Db_NoRecordExists',
                       'options' => array(
                           'table' => 'profile',
                           'field' => 'mail',
                           'adapter' => $this->getAdapter()
                       )
                   )
                ),
            ));

            $inputFilter->add(array(
                'name'     => 'password',
                'required' => true,
                'filters'  => array(
                    array('name' => 'StripTags'),
                    array('name' => 'StringTrim'),
                ),
                'validators' => array(
                   array(
                       'name'    => 'StringLength',
                       'options' => array(
                          'encoding' => 'UTF-8',
                          'min'      => 4,
                          'max'      => 12,
                       ),
                   )
                ),
            ));

            $this->inputFilter = $inputFilter;
        }

        return $this->inputFilter;
    }
    
    public function setAdapter($adapter) {
        $this->adapter = $adapter;
    }
    
    public function getAdapter() {
        return $this->adapter;
    }
    
    public function __toString () {
        return json_encode(array('id' => $this->id,
                                'mail' => $this->mail,
                                'password' => $this->password,
                                'key' => $this->key,
                                'name' => $this->name,
                                'lastname' => $this->lastname));
    }
}