<?php

/* 
 * Copyright (C) 2013 Rafael Barragán Górriz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Profile\Model;

use Zend\Db\TableGateway\TableGateway;

class ProfileTable
{
    protected $tableGateway;

    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
    }

    public function fetchAll()
    {
        $resultSet = $this->tableGateway->select();
        return $resultSet;
    }

    public function getProfile($id)
    {
        $id  = (int) $id;
        $rowset = $this->tableGateway->select(array('id' => $id));
        $row = $rowset->current();
        if (!$row) {
            throw new \Exception("Could not find row $id");
        }
        return $row;
    }
    
    public function getProfileByKey($key) {
        $rowset = $this->tableGateway->select(array('key' => $key));
        $row = $rowset->current();
        if (!$row) {
            throw new \Exception("Could not find row $key");
        }
        return $row;
    }
    
    /**
     * Return key associated to the user or false if not valid login
     * @param type $mail
     * @param type $password
     * @return type String
     */
    public function login($mail, $password) {
        $rowset = $this->tableGateway->select(array('mail' => $mail, 'password' => $password));
        $row = $rowset->current();
        if (!$row) {
            return false;
        }
        
        return $row;
    }
    
    public function saveProfile(Profile $profile)
    {
        $data = array(
            'mail' => $profile->mail,
            'password'  => $profile->password,
            'name' => $profile->name,
            'lastname' => $profile->lastname,
        );

        $id = (int) $profile->id;
        if ($id == 0) {
            $data['key'] = $profile->key;
            $this->tableGateway->insert($data);
        } else {
            $this->tableGateway->update($data, array('id' => $id));
        }
    }

    public function deleteProfile($id)
    {
        $this->tableGateway->delete(array('id' => (int) $id));
    }
    
    public function getTableGateway() {
        return $this->tableGateway;
    }
}